<?php
namespace ReyCore;

defined('ABSPATH') || exit;

/**
 * ReyCore autoloader.
 * Handles dynamically loading classes only when needed.
 *
 * @since 1.0.0
 */
class Autoloader
{

	private static $cached_paths = [];

	/**
	 * Run autoloader.
	 * Register a function as `__autoload()` implementation.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public static function run() {
		spl_autoload_register([__CLASS__, 'autoload']);
	}

	/**
	 *
	 * @param $class_name
	 */
	private static function autoload($class_name) {

		if( 0 !== strpos($class_name, __NAMESPACE__) ) {
			return;
		}

		// Check if we've got it cached and ready.
		if ( isset( self::$cached_paths[ $class_name ] ) && file_exists( self::$cached_paths[ $class_name ] ) ) {
			include_once self::$cached_paths[ $class_name ]; // phpcs:ignore WPThemeReview.CoreFunctionality.FileInclude
			return;
		}

		$file_name = strtolower(
			preg_replace(
				[
					'/\b' . __NAMESPACE__ . '\\\/',
					'/WooCommerce/',
					'/([a-z])([A-Z])/',
					'/_/',
					'/\\\/'
				],
				[
					'',
					'woocommerce',
					'$1-$2',
					'-',
					DIRECTORY_SEPARATOR
				],
				$class_name
			)
		);

		$plugin_dir = plugin_dir_path(__FILE__);

		$sources = [
			'inc',
		];

		foreach ($sources as $source) {

			$file = $plugin_dir . trailingslashit( $source ) . $file_name . '.php';

			if( file_exists( $file ) ) {

				self::$cached_paths[ $class_name ] = $file;

				include_once $file;
			}

		}

	}
}

/**
 * Calling the autoloader to class files
 *
 */
Autoloader::run();
