<?php
namespace ReyCore;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Assets {

	public function __construct(){

		add_action( 'init', [$this, 'init']);

	}

	public function init(){

		add_action( 'reycore/assets/register_scripts', [$this, 'register_scripts']);
		add_action( 'wp_enqueue_scripts', [$this, 'fallback_enqueue_scripts']);
		add_filter( 'reycore/assets/excludes_choices', [$this, 'add_excludes_choices']);

	}

	public function styles(){

		$rtl = reyCoreAssets()::rtl();

		$styles = [
			'reycore-general' => [
				'src'      => REY_CORE_URI . 'assets/css/general-components/general/general' . $rtl . '.css',
				'priority' => 'high',
				'enqueue'  => true,
				'desc' => 'Core General'
			],
			'reycore-header-search-top' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/header-search-top/header-search-top' . $rtl . '.css',
				'deps'    => [],
				'version'   => REY_CORE_VERSION,
				'priority' => 'high'
			],
			'reycore-header-search' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/header-search/header-search' . $rtl . '.css',
				'deps'    => [],
				'version'   => REY_CORE_VERSION,
				'priority' => 'low'
			],
			'reycore-main-menu' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/main-menu/main-menu' . $rtl . '.css',
				'priority' => 'high'
			],
			'reycore-ajax-load-more' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/ajax-load-more/ajax-load-more' . $rtl . '.css',
				'priority' => 'low'
			],

			'reycore-language-switcher' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/language-switcher/language-switcher' . $rtl . '.css',
				'priority' => 'high',
			],
			'reycore-menu-icons' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/menu-icons/menu-icons' . $rtl . '.css',
				'priority' => 'high',
			],
			'reycore-modals' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/modals/modals' . $rtl . '.css',
				'priority' => 'low',
			],
			'reycore-post-social-share' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/post-social-share/post-social-share' . $rtl . '.css',
			],
			'reycore-side-panel' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/side-panel/side-panel' . $rtl . '.css',
				'priority' => 'low',
			],
			'reycore-sticky-social' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/sticky-social/sticky-social' . $rtl . '.css',
				'priority' => 'high',
			],
			'reycore-utilities' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/utilities/utilities' . $rtl . '.css',
				'priority' => 'low',
			],
			'reycore-pass-visibility' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/pass-visibility/pass-visibility' . $rtl . '.css',
				'priority' => 'low',
			],
			'reycore-videos' => [
				'src'     => REY_CORE_URI . 'assets/css/general-components/videos/videos' . $rtl . '.css',
				'priority' => 'high',
			],
			'simple-scrollbar' => [
				'src'     => REY_CORE_URI . 'assets/css/lib/simple-scrollbar.css',
				'deps'      => [],
			],
			'rey-splide' => [
				'src'     => REY_CORE_URI . 'assets/css/lib/splide.css',
				'priority' => 'high',
				'deps'      => [],
			],
			'reycore-slider-components' => [
				'src'     => REY_CORE_URI . 'assets/css/lib/slider-components.css',
				'priority' => 'low',
				'deps'      => [],
			],
		];

		foreach ($styles as $key => $style) {

			if( ! isset($style['deps']) ){
				$styles[$key]['deps'] = function_exists('reyAssets') ? reyAssets()::STYLE_HANDLE : [];
			}

			if( ! isset($style['version']) ){
				$styles[$key]['version'] = REY_CORE_VERSION;
			}
		}

		return $styles;
	}

	public function scripts(){

		return [
			'flying-pages' => [
				'src'     => REY_CORE_URI . 'assets/js/lib/flying-pages.js',
				'deps'    => [],
				'version' => '2.1.2-r',
				'enqueue' => get_theme_mod('perf__enable_flying_scripts', false),
				'defer' => true,
				'localize' => [
					'name' => 'FPConfig',
					'params' => [
						'delay' => 0,
						'ignoreKeywords' => ['wp-admin', 'logout', 'wp-login.php', 'add-to-cart=', 'customer-logout', 'remove_item=', 'apply_coupon=', 'remove_coupon=', 'undo_item=', 'update_cart=', 'proceed=', 'removed_item=', 'added-to-cart=', 'order_again='],
						'maxRPS' => 3,
						'hoverDelay' => 50,
					],
				],
				'plugin' => true
			],
			'animejs' => [
				'src'     => REY_CORE_URI . 'assets/js/lib/anime.min.js',
				'deps'    => ['jquery'],
				'version' => '3.1.0',
				'plugin' => true
			],
			'simple-scrollbar' => [
				'src'     => REY_CORE_URI . 'assets/js/lib/simple-scrollbar.js',
				'deps'    => ['jquery'],
				'version' => '0.4.0',
			],
			'wnumb' => [
				'src'     => REY_CORE_URI . 'assets/js/lib/wnumb.js',
				'deps'    => [],
				'version' => '1.2.0',
			],
			'slick' => [
				'src'     => REY_CORE_URI . 'assets/js/lib/slick.js',
				'deps'    => ['jquery'],
				'version' => '1.8.1',
				'plugin' => true
			],
			'splidejs' => [
				'src'     => REY_CORE_URI . 'assets/js/lib/splide.js',
				'deps'    => [],
				'version' => '3.6.12',
				'plugin' => true,
			],
			'rey-splide' => [
				'src'     => REY_CORE_URI . 'assets/js/general/c-slider.js',
				'deps'    => ['splidejs'],
				'version' => REY_CORE_VERSION,
				'localize' => [
					'name' => 'reyCoreSplideParams',
					'params' => [
						'classes' => [],
						'direction' => is_rtl() ? 'rtl' : 'ltr',
					],
				],
			],
			'scroll-out' => [
				'src'     => REY_CORE_URI . 'assets/js/lib/scroll-out.js',
				'deps'    => ['jquery'],
				'version' => '2.2.3',
				'callback' => 'rey__is_blog_list',
				'plugin' => true
			],
			'reycore-scripts' => [
				'src'      => REY_CORE_URI . 'assets/js/general/c-general.js',
				'deps'     => ['jquery', 'imagesloaded', 'rey-script'],
				'version'  => REY_CORE_VERSION,
				'localize' => [
					'name' => 'reyCoreParams',
					'params' => apply_filters('reycore/script_params', [
						'icons_path' => reycore__icons_sprite_path(),
						'social_icons_path' => reycore__social_icons_sprite_path(),
						'js_params'     => [
							'sticky_debounce' => 200,
							'dir_aware' => false,
							'svg_icons_version' => apply_filters('rey/svg_icon/append_version', false) ? REY_CORE_VERSION : ''
						]
					]),
				],
				'enqueue' => true
			],
			'rey-videos' => [
				'src'     => REY_CORE_URI . 'assets/js/general/c-videos.js',
				'deps'    => ['reycore-scripts'],
				'version' => REY_CORE_VERSION,
			],
			'rey-horizontal-drag' => [
				'src'     => REY_CORE_URI . 'assets/js/general/c-horizontal-drag.js',
				'deps'    => ['reycore-scripts'],
				'version' => REY_CORE_VERSION,
			],
			'reycore-header-search' => [
				'src'     => REY_CORE_URI . 'assets/js/general/c-header-search.js',
				'deps'    => ['reycore-scripts'],
				'version' => REY_CORE_VERSION,
			],
			'reycore-load-more' => [
				'src'     => REY_CORE_URI . 'assets/js/general/c-load-more.js',
				'deps'    => ['reycore-scripts', 'scroll-out'],
				'version' => REY_CORE_VERSION,
			],

			'reycore-modals' => [
				'src'     => REY_CORE_URI . 'assets/js/general/c-modal.js',
				'deps'    => ['reycore-scripts', 'wp-util'],
				'version' => REY_CORE_VERSION,
			],
			'reycore-sticky-global-sections' => [
				'src'     => REY_CORE_URI . 'assets/js/general/c-sticky-global-sections.js',
				'deps'    => ['reycore-scripts'],
				'version' => REY_CORE_VERSION,
			],
			'reycore-sticky' => [
				'src'     => REY_CORE_URI . 'assets/js/general/c-sticky.js',
				'deps'    => ['reycore-scripts', 'imagesloaded'],
				'version' => REY_CORE_VERSION,
			],
			'reycore-delay-js' => [
				'src'     => REY_CORE_URI . 'assets/js/general/c-delay-js.js',
				'deps'    => ['reycore-scripts'],
				'version' => REY_CORE_VERSION,
				'localize' => [
					'name' => 'reyCoreDelayJS',
					'params' => apply_filters('reycore/delay_script_params', [
						'event' => '',
					]),
				],
			],
		];
	}

	function register_scripts( $assets_manager )
	{

		$assets_manager->register_asset('styles', $this->styles());
		$assets_manager->register_asset('scripts', $this->scripts());

		if( is_user_logged_in() ){
			wp_register_style(
				'reycore-frontend-admin',
				REY_CORE_URI . 'assets/css/general-components/frontend-admin/frontend-admin' . $assets_manager::rtl() . '.css',
				[],
				REY_CORE_VERSION
			);
		}

	}



	function fallback_enqueue_scripts()
	{

		if( function_exists('reyAssets') ){
			return;
		}

		$excludes = reyCoreAssets()->get_excludes();

		foreach (['styles', 'scripts'] as $type) {

			$func = 'all_' . $type;

			if( !( is_callable([$this, $func]) && $assets = call_user_func([$this, $func]) ) ){
				continue;
			}

			foreach( $assets as $handle => $asset ){

				$enqueue = false;

				// always enqueue
				if( isset($asset['enqueue']) && $asset['enqueue'] ){
					$enqueue = ! in_array($handle, $excludes, true);
				}

				else {
					// check callback
					if( isset($asset['callback']) ){
						if( is_callable($asset['callback']) && call_user_func($asset['callback']) ){
							$enqueue = true;
						}
					}
				}

				if( $enqueue ){
					call_user_func( [ reyCoreAssets(), 'add_' . $type ], $handle );
				}
			}
		}
	}


	public function add_excludes_choices( $choices ){
		return array_merge($choices, wp_list_filter( $this->styles(), [
			'enqueue' => true
		] ));
	}
}
