<?php
namespace ReyCore\Customizer\Options\General;

if ( ! defined( 'ABSPATH' ) ) exit;

use \ReyCore\Customizer\Controls;

class Layout extends \ReyCore\Customizer\SectionsBase {

	public static function get_id(){
		return 'general-layout';
	}

	public function get_title(){
		return esc_html__('Site Layout', 'rey-core');
	}

	public function get_priority(){
		return 5;
	}

	public function get_icon(){
		return 'site-layout';
	}

	public function help_link(){
		return reycore__support_url('kb/customizer-general-settings/#layout-settings');
	}

	public function controls(){

		$this->add_control( [
			'type'        => 'select',
			'settings'    => 'custom_container_width',
			'label'       => esc_html__( 'Container Width (Desktop)', 'rey-core' ),
			'default'     => 'default',
			'choices'     => [
				'default'   => esc_html__( 'Default', 'rey-core' ),
				'full' => esc_html__( 'Full Width', 'rey-core' ),
				'px'  => esc_html__( 'Pixels (px)', 'rey-core' ),
				'vw' => esc_html__( 'Viewport (vw)', 'rey-core' ),
				// 'pc' => esc_html__( 'Percent (%)', 'rey-core' ),
			],
		] );

		$this->add_control( array(
			'type'        		=> 'slider',
			'settings'    		=> 'container_width_vw',
			'label'       		=> esc_attr__( 'Container Width (vw)', 'rey-core' ),
			'default'     		=> 90,
			'choices'     		=> array(
				'min'  => 50,
				'max'  => 99,
				'step' => 1,
			),
			'transport'   		=> 'auto',
			'active_callback' => [
				[
					'setting'  => 'custom_container_width',
					'operator' => '==',
					'value'    => 'vw',
				],
			],
		));

		$this->add_control( array(
			'type'        		=> 'slider',
			'settings'    		=> 'container_width_px',
			'label'       		=> esc_attr__( 'Container Width (px)', 'rey-core' ),
			'default'     		=> 1440,
			'choices'     		=> array(
				'min'  => 1025,
				'max'  => 2560,
				'step' => 10,
			),
			'transport'   		=> 'auto',
			'output'      		=> [
				[
					'element'  		=> ':root',
					'property' 		=> '--container-max-width',
					'units'    		=> 'px',
				]
			],
			'active_callback' => [
				[
					'setting'  => 'custom_container_width',
					'operator' => '==',
					'value'    => 'px',
				],
			],
		));


		$this->add_control( [
			'type'        => 'dimensions',
			'settings'    => 'site_padding',
			'label' => esc_html__( 'Site Padding', 'rey-core' ),
			'help' => [
				__( 'Will add padding around the site container (<strong>including header, content and footer</strong>). Dont forget to include unit (eg: px, em, rem).', 'rey-core' )
			],
			'default'     => [
				'padding-top'    => '',
				'padding-right'  => '',
				'padding-bottom' => '',
				'padding-left'   => '',
			],
			'choices'     => [
				'labels' => [
					'padding-top'  => esc_html__( 'Top', 'rey-core' ),
					'padding-right' => esc_html__( 'Right', 'rey-core' ),
					'padding-bottom'  => esc_html__( 'Bottom', 'rey-core' ),
					'padding-left' => esc_html__( 'Left', 'rey-core' ),
				],
			],
			'transport' => 'auto',
			'output'    => [
				[
					'element'  => ':root',
					'property' => '--site',
					'units'    => 'px',
				],
			],
			'css_class' => 'dimensions-4-cols',
		] );


		$this->add_control( [
			'type'        => 'dimensions',
			'settings'    => 'content_padding',
			'label'       => esc_html__( 'Content Padding', 'rey-core' ),
			'help' => [
				__( 'Will add padding around the <strong>content</strong> container only. Dont forget to include unit (eg: px, em, rem).', 'rey-core' )
			],
			'default'     => [
				'padding-top'    => '',
				'padding-right'  => '',
				'padding-bottom' => '',
				'padding-left'   => '',
			],
			'choices'     => [
				'labels' => [
					'padding-top'  => esc_html__( 'Top', 'rey-core' ),
					'padding-right' => esc_html__( 'Right', 'rey-core' ),
					'padding-bottom'  => esc_html__( 'Bottom', 'rey-core' ),
					'padding-left' => esc_html__( 'Left', 'rey-core' ),
				],
			],
			'transport'   		=> 'auto',
			'output'      		=> [
				[
					'element'  => ':root',
					'property' => '--content',
					'units'    => 'px',
				],
			],
			'css_class' => 'dimensions-4-cols',
		] );

		$this->add_control( [
			'type'        => 'toggle',
			'settings'    => 'accessibility__hide_btn_focus',
			'label'       => esc_html__( 'Hide Button Focus (Accessibility)', 'rey-core' ),
			'help' => [
				__( 'This will hide the outlines added on buttons when focusing.', 'rey-core' )
			],
			'default'     => true,
			'separator' => 'before',
		] );


	}

}
