<?php
namespace ReyCore\Customizer\Options\Woocommerce;

if ( ! defined( 'ABSPATH' ) ) exit;

use \ReyCore\Customizer\Controls;

class Search extends \ReyCore\Customizer\SectionsBase {

	public static function get_id(){
		return 'woo-search-page';
	}

	public static function part_of(){
		return 'general-search-page';
	}

	public function get_title(){
		return esc_html__('Search Page (Woo)', 'rey-core');
	}

	public function controls(){

		$this->add_control( [
			'type'        => 'multicheck',
			'settings'    => 'search__include',
			'label'       => esc_html__( 'Include taxonomies in search', 'rey-core' ),
			'default'     => [],
			'choices'     => \ReyCore\Customizer\Helper::wc_taxonomies(),
			'help' => [
				esc_html__( 'Please use with attention because it might return too many generic results and will slow down the search query.', 'rey-core' )
			],
			'separator' => 'before',
			'priority' => 60,
		] );

		$this->add_control( [
			'type'        => 'toggle',
			'settings'    => 'search__sku',
			'label'       => esc_html__( 'Search Product SKU', 'rey-core' ),
			'default'     => true,
			'help' => [
				esc_html__( 'May slow down the search.', 'rey-core' )
			],
			'priority'    => 70,
		] );

		$this->add_title( esc_html__('EMPTY RESULTS PAGE', 'rey-core'), [
			'priority' => 80,
		] );

		$this->add_control( [
			'type'        => 'select',
			'settings'    => 'loop_search_empty__gs',
			'label'       => esc_html__( 'Show Global Section', 'rey-core' ),
			'default'     => '',
			'choices'     => \ReyCore\Customizer\Helper::global_sections('generic', ['' => '- None -']),
			'priority' => 90,
		] );

		$this->add_control( [
			'type'        => 'select',
			'settings'    => 'loop_search_empty__mode',
			'label'       => esc_html__( 'Mode', 'rey-core' ),
			'default'     => 'overwrite',
			'choices'     => [
				'overwrite' => esc_html__( 'Overwrite Content', 'rey-core' ),
				'before' => esc_html__( 'Add Before', 'rey-core' ),
				'after' => esc_html__( 'Add After', 'rey-core' ),
			],
			'active_callback' => [
				[
					'setting'  => 'loop_search_empty__gs',
					'operator' => '!=',
					'value'    => '',
				],
			],
			'priority' => 100,
		] );

	}
}
