<?php
namespace ReyCore;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Helper {


	const STATIC_TRANSIENTS = [];

	const QUERY = '_rey_query_';
	const PRODUCTS = '_rey_products_';
	const TERMS = '_rey_terms_';
	const MENU = '_rey_menu_';
	const CACHE_QUERIES = false;

	public function __construct()
	{
		add_action( 'init', [ $this, 'init' ]);
	}

	public function init(){
	}

	public static function get_transient( $transient_name, $cb, $expiration = false ){

		if( ! $transient_name ){
			return;
		}

		if( false !== ($content = get_transient($transient_name)) ){
			return $content;
		}

		if( ! $expiration && isset(self::STATIC_TRANSIENTS[$transient_name]) ){
			$expiration = self::STATIC_TRANSIENTS[$transient_name];
		}

		$content = $cb();

		set_transient($transient_name, $content, $expiration);

		return $content;
	}

	public function clean_transient($post_id){
		if ($post_id === REY_CORE_THEME_NAME) {

			if( function_exists('rey__maybe_disable_obj_cache') ){
				rey__maybe_disable_obj_cache();
			}

			foreach (self::STATIC_TRANSIENTS as $k => $v){
				delete_transient($k);
			}
		}
	}

	public static function get_terms($args = []){

		if( empty($args) ){
			return [];
		}

		if( ! self::CACHE_QUERIES ){
			return get_terms($args);
		}

		$tax = '';

		if( isset($args['taxonomy']) ){
			$tax = $args['taxonomy'];
		}

		$name = self::TERMS . $tax . '_' . md5(wp_json_encode($args));

		return self::get_transient( $name, function() use ($args){
			return get_terms($args);
		}, WEEK_IN_SECONDS);
	}

	public static function get_products_query($args = []){

		if( empty($args) ){
			return [];
		}

		if( ! self::CACHE_QUERIES ){
			return new \WP_Query( $args );
		}

		$name = self::PRODUCTS . md5(wp_json_encode($args));

		// make sure to force
		$args['post_type'] = 'product';

		return self::get_transient( $name, function() use ($args){
			return new \WP_Query( $args );
		}, WEEK_IN_SECONDS);
	}

	public static function get_query($args = []){

		if( empty($args) ){
			return [];
		}

		if( ! self::CACHE_QUERIES ){
			return new \WP_Query( $args );
		}

		$pt = '';

		if( isset($args['post_type']) ){
			$pt = $args['post_type'];
		}

		$name = self::QUERY . $pt . '_' . md5(wp_json_encode($args));

		return self::get_transient( $name, function() use ($args){
			return new \WP_Query( $args );
		}, WEEK_IN_SECONDS);
	}

	public static function wp_nav_menu($args = []){

		if( empty($args) ){
			return [];
		}

		if( ! self::CACHE_QUERIES ){
			return wp_nav_menu( $args );
		}

		$menu_id = '';

		if( isset($args['menu']) ){
			$menu_id = $args['menu'];
		}

		$name = self::MENU . $menu_id . '_' . md5(wp_json_encode($args));

		return self::get_transient( $name, function() use ($args){
			return wp_nav_menu( $args );
		}, WEEK_IN_SECONDS);
	}

	/**
	 * Get a list of all WordPress menus
	 *
	 * @since 1.0.0
	 */
	public static function get_all_menus( $clean = true ){

		$terms = self::get_terms( [
			'taxonomy' => 'nav_menu'
		] );

		if( $clean ){

			$menus = [];
			foreach ($terms as $term) {
				$menus[$term->slug] = $term->name;
			}
			return $menus;
		}

		if( !is_array($terms) ){
			return [];
		}
		else {
			return $terms;
		}
	}

	public static function get_all_image_sizes( $add_default = true ) {
		global $_wp_additional_image_sizes;

		$default_image_sizes = [ 'thumbnail', 'medium', 'medium_large', 'large' ];

		$wp_image_sizes = [];

		foreach ( $default_image_sizes as $size ) {
			$wp_image_sizes[ $size ] = [
				'width' => (int) get_option( $size . '_size_w' ),
				'height' => (int) get_option( $size . '_size_h' ),
				'crop' => (bool) get_option( $size . '_crop' ),
			];
		}

		if ( $_wp_additional_image_sizes ) {
			$wp_image_sizes = array_merge( $wp_image_sizes, $_wp_additional_image_sizes );
		}

		/** This filter is documented in wp-admin/includes/media.php */
		$wp_image_sizes = apply_filters( 'image_size_names_choose', $wp_image_sizes );

		$image_sizes = [];

		if( $add_default ){
			$image_sizes[''] = esc_html__( 'Default', 'rey-core' );
		}

		foreach ( $wp_image_sizes as $size_key => $size_attributes ) {

			$control_title = ucwords( str_replace( '_', ' ', $size_key ) );

			if ( is_array( $size_attributes ) ) {
				$control_title .= sprintf( ' - %d x %d', $size_attributes['width'], $size_attributes['height'] );
			}

			$image_sizes[ $size_key ] = $control_title;
		}

		$image_sizes['full'] = _x( 'Full', 'Image Size Control', 'rey-core' );

		return $image_sizes;
	}

	/**
	 * @param int $number
	 * @return string
	 */
	public static function numberToRoman($number) {
		$map = array('M' => 1000, 'CM' => 900, 'D' => 500, 'CD' => 400, 'C' => 100, 'XC' => 90, 'L' => 50, 'XL' => 40, 'X' => 10, 'IX' => 9, 'V' => 5, 'IV' => 4, 'I' => 1);
		$returnValue = '';
		while ($number > 0) {
			foreach ($map as $roman => $int) {
				if($number >= $int) {
					$number -= $int;
					$returnValue .= $roman;
					break;
				}
			}
		}
		return $returnValue;
	}

	public static function get_video_properties( $video_url ) {

		$provider_match_masks = [
			'youtube' => '/^.*(?:youtu\.be\/|youtube(?:-nocookie)?\.com\/(?:(?:watch)?\?(?:.*&)?vi?=|(?:embed|v|vi|user)\/))([^\?&\"\'>]+)/',
			'vimeo' => '/^.*vimeo\.com\/(?:[a-z]*\/)*([‌​0-9]{6,11})[?]?.*/',
		];

		foreach ( $provider_match_masks as $provider => $match_mask ) {

			preg_match( $match_mask, $video_url, $matches );

			if ( $matches ) {

				$query = [];
				$parts = parse_url($video_url);

				if( isset($parts['query']) && !empty($parts['query']) ){
					parse_str($parts['query'], $query);
					unset($query['v']);
				}

				return [
					'provider' => $provider,
					'video_id' => $matches[1],
					'query' => $query,
				];
			}
		}

		return null;
	}

	public static function get_embed_video( $args = [] ) {

		$args = wp_parse_args($args, [
			'url'      => '',
			'style'    => '',
			'class'    => '',
			'autoplay' => true,
			'id'       => ''
		]);

		if( ! $args['url'] ){
			return;
		}

		$autoplay = (int) $args['autoplay'];
		$video_properties = self::get_video_properties( $args['url'] );

		if( isset($video_properties['provider']) ) {

			$src = '';

			if( 'youtube' === $video_properties['provider'] ){

				$query_args = array_merge([
					'autoplay' => $autoplay,
					'rel'      => 0,
					'mute'     => $autoplay,
					'loop'     => 1,
					'playlist' => $video_properties['video_id'],
				], $video_properties['query']);

				$src = sprintf('//www.youtube.com/embed/%s?%s', $video_properties['video_id'], http_build_query($query_args));
			}
			else if( 'vimeo' === $video_properties['provider'] ){

				$query_args = array_merge([
					'autoplay' => $autoplay,
					'muted'    => $autoplay,
					'loop'     => 1,
				], $video_properties['query']);

				$src = sprintf('//player.vimeo.com/video/%s?%s', $video_properties['video_id'], http_build_query($query_args));
			}

			$video = '<iframe allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" data-video="'. $src .'" frameborder="0" allowfullscreen></iframe>';

		}

		// self hosted
		else {

			if( $autoplay ){
				$attrs = 'muted="muted" autoplay="autoplay"';
			}

			$video = sprintf('<video loop="loop" %s playsinline="playsinline" data-video="'. $args['url'] .'"></video>', $attrs);
		}

		if( $video ){
			return sprintf('<div class="embed-responsive embed-responsive-%4$s %2$s" style="%3$s">%1$s</div>', $video, $args['class'], $args['style'], apply_filters('reycore/embed_video_ratio', '16by9', $args) );
		}

	}

	/**
	 * Get File Path
	 *
	 * Will return the file path starting with the plugin directory for the given path
	 *
	 * @param string $path The path that will be appended to the plugin path
	 *
	 * @return string
	 */
	public static function get_file_path( $path = '' ) {
		return REY_CORE_DIR . $path;
	}

	// Normalize class name
	public static function fix_class_name( $name, $prefix = '', $suffix = '' ){
		$class_name = str_replace( '/', '\\', $name );
		$class_prefix = $prefix ? $prefix . '\\' : '';
		$class_suffix = $suffix ? '\\' . $suffix : '';
		return __NAMESPACE__ . '\\' . $class_prefix . $class_name . $class_suffix;
	}

	public static function is_rest_api_request() {
		if ( empty( $_SERVER['REQUEST_URI'] ) ) {
			// Probably a CLI request
			return false;
		}

		$rest_prefix         = trailingslashit( rest_get_url_prefix() );
		$is_rest_api_request = strpos( $_SERVER['REQUEST_URI'], $rest_prefix ) !== false;

		return apply_filters( 'is_rest_api_request', $is_rest_api_request );
	}

}
