<?php
namespace ReyCore\Modules;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Base {

	/**
	 * Holds the modules that are registered
	 */
	private $modules = [];

	/**
	 * All modules that are included
	 */
	private $all_modules = [];

	public function __construct(){

		// problems with Custom Templates Post Type module
		// if( isset($_REQUEST['action']) && 'heartbeat' === reycore__clean($_REQUEST['action']) ){
		// 	return;
		// }

		$this->module_manager = new ModuleManager();

		$this->load_modules();

	}

	public function load_modules(){

		foreach ( self::get_modules_list() as $module_id) {

			// Normalize class name
			$class_name = ucwords( str_replace( '-', ' ', $module_id ) );
			$class_name = str_replace( ' ', '', $class_name );
			$class_name = \ReyCore\Helper::fix_class_name($class_name, 'Modules', 'Base');

			if( ! class_exists($class_name) ){
				trigger_error('Module classname is incorrect and cannot be loaded.', E_USER_NOTICE);
				continue;
			}

			if( ! $class_name::can_run() ){
				continue;
			}

			// get module config
			$module_data = $class_name::get_module_data();

			$this->all_modules[ $module_id ] = $module_data;

			// prevent loading if disabled
			if( ! $this->module_manager->is_enabled( $module_id ) ){
				continue;
			}

			/**
			 * Legacy way of disabling modules
			 * @todo remove in 2.5.0
			 */
			if( defined('REY_CORE_MODULES_EXCLUDE') ){

				if ( WP_DEBUG ) {
					trigger_error( '"REY_CORE_MODULES_EXCLUDE" constant is deprecated. Use Module Manager instead.', E_USER_DEPRECATED );
				}

				if( is_array(REY_CORE_MODULES_EXCLUDE) && in_array( $module_id, REY_CORE_MODULES_EXCLUDE, true ) ){
					continue;
				}

			}

			// assign module
			$this->modules[ $module_id ] = new $class_name();

		}

	}

	public function get_all_modules( $sorted = false ){

		if( $sorted ){
			usort($this->all_modules, function ($a, $b)	{
				return strcmp($a['title'], $b['title']);
			});
		}

		return $this->all_modules;
	}

	public function get_module( $module = '' ){

		if( $module && isset($this->modules[ $module ]) ){
			return $this->modules[ $module ];
		}

	}

	public function get_modules_list(){

		return [
			'after-atc-popup',
			'ajax-filters',
			'ajax-variables-popup',
			'archive-bottom-desc',
			'atc-button-icon',
			'blog-teasers',
			'brands',
			'buy-now-button',
			'cards',
			'compare',
			'cookie-notice',
			'custom-sidebars',
			'custom-templates',
			'customizer-search',
			'discount-badge',
			'elementor-acf',
			'elementor-animations',
			'estimated-delivery',
			'extra-variation-images',
			'footer-reveal',
			'gallery-three-sixty',
			'header-fixed',
			'inline-search',
			'loop-product-skin-cards',
			'loop-product-skin-iconized',
			'loop-product-skin-proto',
			'loop-product-skin-rigo',
			'mega-menus',
			'mini-cart-bubble',
			'mini-cart-carousel',
			'mini-cart-recents',
			'mini-cart-shipping-bar',
			'offcanvas-panels',
			'order-refunds',
			'product-badges',
			'product-quantity',
			'product-before-after',
			'product-grid-custom-tax',
			'product-stretch',
			'product-video',
			'product-teasers',
			'pdp-custom-tabs',
			'pdp-tabs-accordion',
			'price-features',
			'price-in-atc',
			'product-page-after-content',
			'products-per-page',
			'related-products',
			'store-notice',
			'quickview',
			'request-quote',
			'scheduled-sales',
			'sticky-add-to-cart',
			'scroll-to-top',
			'variation-swatches',
			'view-switcher',
			'wc-product-attributes-widget',
			'wishlist',
		];

	}

}
