<?php
namespace ReyCore\Modules;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class ModuleManager extends \ReyCore\ManagerBase
{

	const DB_OPTION = 'reycore-disabled-modules';

	public function __construct()
	{
		if( ! reycore__get_props('modules_manager') ){
			return;
		}

		parent::__construct();
	}

	public function get_id(){
		return 'modules';
	}

	public function get_menu_title(){
		return __( 'Modules Manager', 'rey-core' );
	}

	public function set_page_config(){
		return [
			'title' => esc_html__('Modules Manager', 'rey-core'),
			'description' => __('Use this tool to control what modules and features can run in your site. Disabling a module will prevent any trace of its code to run.<br>If you\'re not sure if a certain module is actually used inside the site, you can use the Scan function on the right side.', 'rey-core'),
			'singular_item' => esc_html__('Module', 'rey-core'),
			'plural_item' => esc_html__('Modules', 'rey-core'),
			'not_in_use' => esc_html__('Not in use. Safe to deactivate module.', 'rey-core'),
		];
	}

	public function get_default_disabled_items(){
		return apply_filters('reycore/modules/disabled_defaults', [
			'product-grid-custom-tax',
		] );
	}

	public function prepare_items(){

		$categories = [
			'elementor' => 'Elementor',
			'woocommerce' => 'WooCommerce',
			'frontend' => 'Frontend',
			'misc' => 'Miscellaneous',
		];

		$groups = [];

		$modules_instance = \ReyCore\Plugin::instance()->modules;

		foreach ( $modules_instance->get_all_modules( true ) as $module_id => $module) {

			if( ! $module['show_in_manager'] ){
				continue;
			}

			$item_category = $module['categories'][0];

			$groups[ $item_category ]['category'] = $categories[ $item_category ];
			$groups[ $item_category ]['items'][$module_id] = $module;

		}

		$this->groups = $groups;

	}

	public function ajax__scan_unused( $data ){

		if ( ! current_user_can('install_plugins') ) {
			return [
				'errors' => [ 'Operation not allowed!' ]
			];
		}

		if( ! isset($data['item']) ){
			return [
				'errors' => [ 'Item not passed!' ]
			];
		}

		// if for some reason the module instance can't be retrieved
		// mark it as in use
		if( ! ($module = \ReyCore\Plugin::instance()->modules->get_module($data['item'])) ){
			return true;
		}

		return $module->module_in_use();

	}

	public function get_all_items(){
		return \ReyCore\Plugin::instance()->modules->get_modules_list();
	}

}
