<?php
namespace ReyCore\Modules\VariationSwatches;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Frontend
{

	public $loop_variations;

	public static $scrips_loaded;

	public $args = [];

	private $swatch_html;

	public function __construct()
	{
		add_action( 'reycore/assets/register_scripts', [$this, 'register_assets']);
		add_action( 'wp_enqueue_scripts', [$this, 'enqueue_scripts']);
		add_action( 'reycore/woocommerce/product_page/scripts', [$this, 'enqueue_scripts']);
		add_action( 'reycore/module/after-atc-popup/scripts', [$this, 'load_scripts']);
		add_filter( 'woocommerce_dropdown_variation_attribute_options_html', [$this, 'variation_dropdown'], 100, 2);
		add_filter( 'woocommerce_dropdown_variation_attribute_options_args', [$this, 'variation_dropdown_args'], 100, 2);
		add_filter( 'woocommerce_ajax_variation_threshold', [$this, 'ajax_variation_threshold'], 10, 2 );
		add_filter( 'reycore/woocommerce/variations/variation_attribute_data', [$this, 'variation_attribute_data'] );
		add_action( 'reycore/woocommerce/variations/catalog/render_single', [$this, 'catalog_render_single_swatch'], 10, 2);
		add_action( 'reycore/woocommerce/variations/catalog/render_all', [$this, 'catalog_render_all_swatches'], 10, 2);
		add_filter( 'post_class', [$this, 'product_classes'], 20 );
		add_filter( 'woocommerce_reset_variations_link', [$this, 'reset_variations_link']);
		add_action( 'reycore/ajax/register_actions', [ $this, 'register_actions' ] );

	}

	/**
	 * Register Ajax Actions
	 *
	 * @param object $ajax_manager
	 * @return void
	 */
	public function register_actions( $ajax_manager ){
		$ajax_manager->register_ajax_action( 'variations_load_catalog', [$this, 'ajax__lazy_load_catalog'], 3 );
	}

	public function get_product(){

		$product = null;

		if ( ! empty( $GLOBALS['product'] ) ) {
			// Product was null so pull from global.
			$product = $GLOBALS['product'];
		}

		if ( $product && ! is_a( $product, 'WC_Product' ) ) {
			// Make sure we have a valid product, or set to false.
			$product = wc_get_product( $product );
		}

		return $product;
	}

	/**
	 * Product page load assets
	 *
	 * @return void
	 */
	public function enqueue_scripts(){

		if( Base::$settings['optimized_assets_load'] ){
			return;
		}

		if( ! reycore_wc__is_product() ){
			return;
		}

		$product = $this->get_product();

		if ( ! ($product && 'variable' === $product->get_type()) ) {
			return;
		}

		self::load_scripts();
	}

	public static function get_asset_handle( $type ){
		return Base::ASSET_HANDLE . '-' . $type;
	}

	/**
	 * Load frontend CSS and JS
	 *
	 * @return void
	 */
	public static function load_scripts(){

		if( self::$scrips_loaded ){
			return;
		}

		wp_enqueue_script( 'wc-add-to-cart-variation' ); // can't load it dynamically

		reyCoreAssets()->add_scripts(Base::ASSET_HANDLE);

		reyCoreAssets()->add_styles(Base::ASSET_HANDLE);

		self::$scrips_loaded = true;
	}

	/**
	 * Register mod assets
	 *
	 * @return void
	 */
	public function register_assets($assets){

		$assets->register_asset('styles', [
			Base::ASSET_HANDLE => [
				'src'     => Base::get_path( basename( __DIR__ ) ) . '/frontend-style.css',
				'deps'    => [],
				'version'   => REY_CORE_VERSION,
			]
		]);

		$assets->register_asset('scripts', [
			Base::ASSET_HANDLE => [
				'src'     => Base::get_path( basename( __DIR__ ) ) . '/frontend-script.js',
				'deps'    => ['rey-script', 'reycore-scripts', 'reycore-woocommerce'],
				'version'   => REY_CORE_VERSION,
			]
		]);

	}

	public function can_add_swatches(){
		return apply_filters('reycore/variation_swatches/support', true);
	}

	public function variation_dropdown_args( $args ){

		if( ! $this->can_add_swatches() ){
			return $args;
		}

		global $product;

		// replace id html attribute to be unique
		if( $product && $product_id = $product->get_id() ){
			$args['id'] = $args['attribute'] . $product_id;
		}

		$attribute_name = str_replace('pa_', '', $args['attribute']);

		if( $attribute_name === get_theme_mod('pdp_swatches__update_gallery', '') ){
			$args['class'] = '--update';
		}

		return $args;
	}

	/**
	 * Filter the variation dropdown, hide it and add html
	 *
	 * @param string $html
	 * @param array $args
	 * @return string
	 */
	public function variation_dropdown($html, $args){

		if( ! $this->can_add_swatches() ){
			return $html;
		}

		if( ! ($this->swatch_html = $this->get_swatch_list_output($args)) ){
			return $html;
		}

		$style_search = '<select ';
		$style_replace = 'style="display:none" ';

		$html = str_replace($style_search, $style_search . $style_replace, $html);

		return $html . $this->swatch_html;
	}

	/**
	 * Outputs swatches list
	 *
	 * @param array $attribute
	 * @return string
	 */
	public function get_swatch_list_output( $args ){

		if( empty($args['options']) ){
			return;
		}

		if( ! ($taxonomy = $args['attribute']) ){
			return;
		}

		$attribute = [];

		$custom_attribute = false;

		$attribute_id = wc_attribute_taxonomy_id_by_name($taxonomy);

		if( $attribute_id && 0 !== strpos( $taxonomy, 'pa_') ){
			$attribute_id = false;
		}

		// custom attributes
		if( ! $attribute_id ){

			if( get_theme_mod('pdp_swatches__custom_use_button', false) ){

				$attribute = [
					'name'         => $taxonomy,
					'slug'         => sanitize_title( $taxonomy ),
					'type'         => Base::instance()->get_default_swatch(),
					'order_by'     => false,
					'has_archives' => false,
				];

				$custom_attribute = true;
			}

			// bail bc. there's no attribute data
			else {
				return;
			}
		}
		else {
			$attribute = (array) wc_get_attribute($attribute_id);
		}

		if( empty($attribute) ){
			return;
		}

		if ( ! array_key_exists( $attribute['type'], Base::instance()->get_swatches_list() ) ) {
			return;
		}

		if( Base::$settings['optimized_assets_load'] ){
			self::load_scripts();
		}

		$list_args = $args;
		$list_args['attribute_data'] = $attribute;
		$list_args['custom_attribute'] = $custom_attribute;

		return Base::instance()->get_swatches( $attribute['type'] )->render_list( $list_args );
	}

	function ajax_variation_threshold( $threshold, $product ){

		if( $custom = absint(get_theme_mod('pdp_swatches__ajax_threshold', 50)) ){
			return $custom;
		}

		return $threshold;
	}

	function variation_attribute_data( $data ){

		if( ! $this->can_add_swatches() ){
			return $data;
		}

		$attribute_id = wc_attribute_taxonomy_id_by_name($data['taxonomy_name']);
		$attribute = (array) wc_get_attribute($attribute_id);

		if ( array_key_exists( $attribute['type'], Base::instance()->get_swatches_list() ) ) {
			$data['type'] =  $attribute['type'];
		}

		return $data;
	}

	public function get_transient_name($product_id){
		return \ReyCore\WooCommerce\Tags\VariationsLoop::transient_name( $product_id );
	}

	public function ajax__lazy_load_catalog( $data ){
	}

	protected function get_catalog_data( $product, $tax = '' ){

		$data = false;

		$product_id = $product->get_id();

		if( Base::$settings['loop_cache'] ){
			$data = get_transient( $this->get_transient_name($product_id) );
		}

		if( false === $data ){

			$data = '';

			$attributes = $product->get_variation_attributes();

			if( ! empty($attributes) ){

				$attr_output = '';

				foreach ( $attributes as $attribute_name => $options ) :

					if( $tax && $attribute_name !== $tax ){
						continue;
					}

					ob_start();

					if( ! $tax && Base::$settings['show_all_labels'] ){
						printf('<div class="rey-swatchList-label"><label for="%s">%s</label></div>',
							esc_attr( sanitize_title( $attribute_name ) ),
							wc_attribute_label( $attribute_name )
						); // WPCS: XSS ok.
					}

					$dropdown_args = [
						'options'   => $options,
						'attribute' => $attribute_name,
						'product'   => $product,
					];

					$options_count = count($options);

					if( $tax ){
						if( ($limit = get_theme_mod('woocommerce_loop_variation_limit', 0)) && $limit < $options_count ){
							$dropdown_args['limit'] = $limit;
							$dropdown_args['limit_diff'] = $options_count - $limit;
							$dropdown_args['options'] = array_slice($options, 0, $limit);
						}
					}

					wc_dropdown_variation_attribute_options( $dropdown_args );

					$attr_output .= ob_get_clean();

				endforeach;

				if( $attr_output ){
					$data = sprintf('<form class="variations_form cart --catalog" action="%1$s" method="post" enctype="multipart/form-data" data-product_id="%2$s" data-product_variations="%3$s" data-atc-text="%4$s"><div class="variations">%5$s</div></form>',
						esc_url( apply_filters( 'woocommerce_add_to_cart_form_action', $product->get_permalink() ) ),
						absint( $product->get_id() ),
						wc_esc_json( wp_json_encode( $product->get_available_variations() ) ),
						esc_attr( $product->single_add_to_cart_text() ),
						$attr_output
					);
				}
			}

			if( Base::$settings['loop_cache'] ){
				set_transient( $this->get_transient_name($product_id), $data, MONTH_IN_SECONDS );
			}

		}

		return $data;
	}

	public function catalog_render( $product, $tax = '' ){

		// $over_threshold = count( $product->get_children() ) >= apply_filters( 'woocommerce_ajax_variation_threshold', 30, $product );
		$over_threshold = false;

		$data = false;

		if( ! $over_threshold ){
			$data = $this->get_catalog_data( $product, $tax );
		}

		if( $data || $over_threshold ){

			$position = get_theme_mod('woocommerce_loop_variation_position', 'after');
			$all = 'all_attributes' === $this->loop_variations->selected_attribute_taxonomy;

			$html_list_classes = [
				'rey-productVariations2',
				($all) ? '--all-attr' : '--single-attr',
				$over_threshold ? '--ajax' : '',
			];

			if(
				in_array($position, ['first', 'before'], true) &&
				! $all &&
				get_theme_mod('woocommerce_loop_variation__side', false)
			 ){
				$html_list_classes[] = '--side';
			}

			$html_list_attributes = [
				'data-attribute' => esc_attr( $this->loop_variations->selected_attribute_taxonomy ),
				'data-position' => esc_attr( get_theme_mod('woocommerce_loop_variation_position', 'after') ),
			];

			$html_list_attributes['class'] = esc_attr( implode(' ', apply_filters('reycore/variation_swatches/catalog_classes', $html_list_classes) ) );

			printf('<div %s>%s</div>', reycore__implode_html_attributes($html_list_attributes),  $data);

			self::load_scripts();

		}
	}

	function catalog_render_all_swatches( $product, $rey_catalog_variations ){
		if( ! $this->can_add_swatches() ){
			return;
		}
		$this->loop_variations = $rey_catalog_variations;
		$this->catalog_render( $product );
	}

	function catalog_render_single_swatch( $product, $rey_catalog_variations ){
		if( ! $this->can_add_swatches() ){
			return;
		}
		$this->loop_variations = $rey_catalog_variations;
		$this->catalog_render( $product, $this->loop_variations->selected_attribute_taxonomy_data['taxonomy_name'] );
	}

	/**
	 * Adds custom classes to the product element (div/li)
	 *
	 * @param array $classes
	 * @return array
	 */
	function product_classes( $classes ){

		if( ! $this->can_add_swatches() ){
			return $classes;
		}

		$product = $this->get_product();

		if ( ! $product ) {
			return $classes;
		}

		$classes['swatches'] = 'rey-swatches';
		$classes['show_selected'] = get_theme_mod('pdp_swatches__selected_name', false) ? 'rey-swatches--show-selected-name' : '';
		$classes['stretch_labels'] = get_theme_mod('pdp_swatches__stretch_label', false) ? 'rey-swatches--stretch-labels' : '';

		if( '' !== get_theme_mod('pdp_swatches__update_gallery', '') ){
			$classes['update_gallery'] = '--swatch-update-gallery';
		}

		return $classes;
	}

	/**
	 * Reset button
	 *
	 * @since 2.2.2
	 **/
	function reset_variations_link( $html )
	{

		if( ! $this->swatch_html ){
			return $html;
		}

		return '<a class="rey-resetVariations --hidden" href="#">' . esc_html__( 'Clear', 'woocommerce' ) . '</a>';
	}

	public static function extract_custom_image($data){

		if( ! $data ){
			return;
		}

		$custom_image_url = false;

		if( isset($data['url']) && ($img_url = $data['url']) ){
			$custom_image_url = $img_url;
		}

		else if( isset($data['id']) && ($img_id = $data['id']) ) {
			$custom_image_url = wp_get_attachment_url($img_id);
		}

		if( $custom_image_url ){
			return sprintf('background-image:url(%s);', $custom_image_url );
		}

	}

}
