<?php
namespace ReyCore;

if (!defined('ABSPATH')) exit; // Exit if accessed directly

class Version {

	/**
	 * Option name for Rey Core plugin version.
	 * Used to store the last version.
	 */
	const VERSION = 'reycore_version';

	public function __construct(){
		add_action( 'init', [$this, 'init'] );
		add_action( 'rey/after_update/core', [ $this, 'after_core_update' ] );
	}

	public function init(){

		// bail if version DB exists
		if( get_option(self::VERSION) ){
			return;
		}

		// Fires after installation,
		// on a clean site
		do_action('reycore/clean_setup');

		// If DB version is missing,
		// force set it (once)
		$this->set_db_version();

	}

	public function set_db_version(){
		update_option(self::VERSION, REY_CORE_VERSION);
	}

	/**
	 * Performs after Rey Core plugin update.
	 * Renews the plugin version DB option
	 *
	 * @param array $options
	 * @return void
	 */
	public function after_core_update(){

		// renew the last version
		$this->set_db_version();

	}

	/**
	 * Retrieve the last DB version
	 *
	 * @param boolean $major
	 * @return string
	 */
	public static function get_last_version( $major = false ){

		$last_version = get_option(self::VERSION);

		if( ! $last_version ){
			$last_version = '2.3.0'; // since this was added
		}

		if( $major ){
			$v = array_map('absint', explode( '.', $last_version ));
			return sprintf('%d.%d', $v[0], $v[1]);
		}

		return $last_version;
	}

	/**
	 * Retrieve the current major x.x version
	 *
	 * @param boolean $last
	 * @return string
	 */
	public static function get_current_major_version( $include_minor = false ){
		$v = array_map('absint', explode( '.', REY_CORE_VERSION ));

		if( $include_minor ){
			return sprintf('%d.%d.0', $v[0], $v[1]);
		}

		return sprintf('%d.%d', $v[0], $v[1]);
	}



}
