<?php
namespace ReyCore\WooCommerce\Tags;

// exit if accessed directly
if( ! defined( 'ABSPATH' ) ) exit;

class VariationsLoop
{
	const TRANSIENT_PRODUCT_VARIATIONS = 'rey_product_variations_';

	public function __construct()
	{
		add_action( 'reycore/customizer/control=woocommerce_loop_variation', [ $this, 'customizer__add_attributes_choices' ], 11, 2 );
		add_action( 'reycore/customizer/control=pdp_swatches__update_gallery', [ $this, 'customizer__add_attributes_choices' ], 11, 2 );

		// Clean transients
		add_action( 'customize_save_woocommerce_loop_variation', [$this, 'customizer__clear_transients']);
		add_action( 'customize_save_woocommerce_loop_variation_limit', [$this, 'customizer__clear_transients']);
		add_action( 'woocommerce_delete_product_transients', [$this, 'clear_product_transient']);
		add_action( 'save_post', [$this, 'clear_product_transient'] );
		add_action( 'woocommerce_attribute_updated', [ $this, 'clean_db_transients' ] );
		add_action( 'woocommerce_attribute_deleted', [$this, 'clean_db_transients'] );

	}

	function customizer__add_attributes_choices( $control_args, $section ){

		$current_control = $section->get_control($control_args['settings']);

		$current_control['choices'] = array_merge($current_control['choices'], reycore_wc__get_attributes_list());

		$section->update_control( $current_control );
	}

	function clean_db_transients(){

		global $wpdb;

		$like_main = '%transient_' . $wpdb->esc_like( self::TRANSIENT_PRODUCT_VARIATIONS ) . '%';
		$like_timeout = '%transient_timeout_' . $wpdb->esc_like( self::TRANSIENT_PRODUCT_VARIATIONS ) . '%';
		$query = $wpdb->prepare( "DELETE FROM {$wpdb->options} WHERE option_name LIKE %s OR option_name LIKE %s ", $like_main, $like_timeout );

		return $wpdb->query($query);
	}

	/**
	 * Clear some transients on Customizer Publish
	 *
	 * @since 1.0.0
	 */
	function customizer__clear_transients( $setting )
	{

		if( ! current_user_can('administrator') ){
			return;
		}

		if( ! method_exists($setting, 'value') ){
			return;
		}

		$setting_data = $setting->id_data();
		// old
		$value = $setting->value();
		// new
		$post_value = $setting->post_value();

		$clear = false;

		if( $value === $post_value ){
			return;
		}

		if( ! isset($setting_data['base']) ){
			return;
		}

		if( 'woocommerce_loop_variation' === $setting_data['base'] ){
			if( $post_value !== 'disabled' ){
				$clear = true;
			}
		}

		elseif( 'woocommerce_loop_variation_limit' === $setting_data['base'] ){
			$clear = true;
		}

		if( $clear ){
			$this->clean_db_transients();
		}
	}

	public static function transient_name( $product_id, $v2 = true ){
		return self::TRANSIENT_PRODUCT_VARIATIONS . ($v2 ? '_v2_' : '') . $product_id;
	}

	function clear_product_transient( $post_id = 0 ){

		if ( $post_id === 0 ) {
			$this->clean_db_transients();
		}

		if ( $post_id > 0 ) {

			if( function_exists('rey__maybe_disable_obj_cache') ){
				rey__maybe_disable_obj_cache();
			}

			delete_transient( self::transient_name($post_id) );
			delete_transient( self::transient_name($post_id, false) );

		}
	}

}
